﻿using NewsSystem.Contracts;
using System;
using System.Data;
using System.Data.Entity;
using System.Linq;

namespace NewsSystem.Repositories
{
    public class BaseRepository<T> : IRepository<T> where T: class
    {
        public BaseRepository(DbContext dbContext)
        {
            if (dbContext == null)
            {
                throw new ArgumentNullException("dbContext");
            }

            this.DbContext = dbContext;
            this.DbSet = dbContext.Set<T>();
        }

        public DbContext DbContext { get; set; }

        public DbSet<T> DbSet { get; set; }

        public virtual IQueryable<T> GetAll()
        {
            return this.DbSet;
        }

        public virtual T Get(int id)
        {
            var entity = this.DbSet.Find(id);
            return entity;
        }

        public virtual void Add(T entity)
        {
            var dbEntityEntry = this.DbContext.Entry(entity);
            if (dbEntityEntry.State == EntityState.Detached)
            {
                this.DbSet.Add(entity);
            }
            else
            {
                dbEntityEntry.State = EntityState.Added;
            }
        }

        public virtual void Update(int id, T entity)
        {
            var dbEntityEntry = this.DbContext.Entry(entity);
            if (dbEntityEntry.State == EntityState.Detached)
            {
                this.DbSet.Attach(entity);
            }

            dbEntityEntry.State = EntityState.Modified;
        }

        public virtual void Delete(T entity)
        {
            this.DbSet.Remove(entity);
        }

        public virtual void Delete(int id)
        {
            var entity = this.Get(id);
            if (entity != null)
            {
                this.Delete(entity);
            }
        }
    }
}
